from dataclasses import dataclass
from datetime import datetime, time

tz_datetime = datetime


@dataclass(frozen=True)
class TimeDelta:
    hr: int = 0
    min: int = 0


@dataclass(frozen=True)
class MilitaryTime:
    def __init__(self, time):
        hr, min = divmod(time, 100)
        assert 0 <= min < 60
        assert 0 <= hr < 24

        object.__setattr__(self, 'hr', hr)
        object.__setattr__(self, 'min', min)

    @classmethod
    def from_hr_min(cls, hr: int, min: int):
        return cls(hr*100 + min)

    @classmethod
    def from_string(cls, military_time_string):
        return cls(int(military_time_string))

    @classmethod
    def extract_from_datetime(cls, dt: datetime):
        return cls.from_hr_min(dt.hour, dt.minute)

    @property
    def time(self):
        return self.hr * 100 + self.min

    def to_time(self, tzinfo) -> time:
        return time(hour=self.hr, minute=self.min, tzinfo=tzinfo)

    def __str__(self):
        return f'{self.hr:02}{self.min:02}'

    def __add__(self, other: TimeDelta):
        min = (self.min + other.min) % 60
        additional_hrs = (self.min + other.min) // 60
        hr = (self.hr + other.hr + additional_hrs) % 24
        return MilitaryTime.from_hr_min(hr, min)

    def __sub__(self, other: TimeDelta):
        min = (self.min - other.min) % 60
        additional_hrs = (self.min - other.min) // 60
        hr = (self.hr - other.hr + additional_hrs) % 24
        return MilitaryTime.from_hr_min(hr, min)

    # NOTE: Defining __init__ on a dataclass one has to use __setattr__ to set
    # properties. Now this has an effect that the implicit __hash__ created by
    # the dataclass decorator ignores them.
    def __hash__(self):
        return hash((self.hr, self.min))



import pytest

from shuttlis.time import MilitaryTime, TimeDelta, WeekDay


@pytest.mark.parametrize('hr,min', [(1, 20), (23, 59), (0, 0)])
def test_military_time_constructor(hr, min):
    mt = MilitaryTime.from_hr_min(hr, min)
    assert 4 == len(str(mt))
    assert hr == mt.hr
    assert min == mt.min


@pytest.mark.parametrize('hr,min', [(24, 00), (12, 61)])
def test_military_time_constructor_fails_when_hr_or_min_are_out_of_range(hr, min):
    with pytest.raises(AssertionError) as e:
        MilitaryTime.from_hr_min(hr, min)


@pytest.mark.parametrize('mt,td,mtr', [
    (MilitaryTime(1320), TimeDelta(1, 20), MilitaryTime(1440)),
    (MilitaryTime(1320), TimeDelta(0, 40), MilitaryTime(1400)),
])
def test_military_time_add(mt, td, mtr):
    assert mtr == mt + td


@pytest.mark.parametrize('mt,td,mtr', [
    (MilitaryTime(1320), TimeDelta(1, 20), MilitaryTime(1200)),
    (MilitaryTime(1320), TimeDelta(0, 40), MilitaryTime(1240)),
])
def test_military_time_sub(mt, td, mtr):
    assert mtr == mt + td


def test_weekday_eq():
    assert WeekDay.MONDAY == WeekDay.MONDAY


@pytest.mark.parametrize('first,snd', [
    (WeekDay.MONDAY, WeekDay.TUESDAY),
    (WeekDay.MONDAY, WeekDay.SUNDAY),
])
def test_weekday_lt(first, snd):
    assert first < snd


def test_weekday_str_conversion():
    assert str(WeekDay.SUNDAY) == 'SUNDAY'


def test_time_delta_with_only_minutes():
    assert MilitaryTime(1359) == MilitaryTime(1300) + TimeDelta(min=59)


def test_hash_of_military_times():
    assert 2 == len({MilitaryTime(900), MilitaryTime(1000)})


def test_hash_of_same_military_time():
    assert 1 == len({MilitaryTime(900), MilitaryTime(900)})



from setuptools import setup, find_packages

setup(
    name='pyshuttlis',
    version='0.1.3',
    description='Utilities',
    url='https://github.com/shuttl-tech/pyshuttlis',
    author='Shuttl',
    author_email='sherub.thakur@shuttl.com',
    license='MIT',
    packages=find_packages(),
    classifiers=["Programming Language :: Python :: 3.7"],
    install_requires=['boto3'],
)



import calendar
from datetime import datetime
from enum import Enum
from functools import total_ordering


@total_ordering
class WeekDay(Enum):
    MONDAY = 0
    TUESDAY = 1
    WEDNESDAY = 2
    THURSDAY = 3
    FRIDAY = 4
    SATURDAY = 5
    SUNDAY = 6

    @classmethod
    def extract_from_datetime(cls, dt: datetime):
        return cls[calendar.day_name[dt.weekday()].upper()]

    @classmethod
    def from_string(cls, weekday: str):
        return cls[weekday.upper()]

    def __str__(self):
        return str(self.name)

    def __lt__(self, other):
        return self.value < other.value

